from glLibLocals import *
from glLibTexturing import glLibTexture

def MTL(filename,mipmapping,filtering):
    contents = {}
    mtl = None
    for line in open(filename, "r"):
        if line.startswith('#'): continue
        values = line.split()
        if not values: continue
        if values[0] == 'newmtl':
            mtl = contents[values[1]] = {}
        elif mtl is None:
            raise ValueError, "mtl file doesn't start with newmtl stmt"
        elif values[0] == 'map_Kd':
            mtl[values[0]] = values[1]
            surf = pygame.image.load(mtl['map_Kd'])
            mtl['texture_Kd'] = glLibTexture(surf,[0,0,surf.get_width(),surf.get_height()],GLLIB_RGBA,mipmapping,filtering)
        else:
            mtl[values[0]] = map(float, values[1:])
    return contents
 
def OBJ(filename,mipmapping,filtering):
    vertices = []
    normals = []
    texcoords = []
    faces = []

    filename = filename.split("/")
    mtl = MTL(os.path.join(*filename[:-1]+[filename[-1][:-4]+".mtl"]),mipmapping,filtering)

    material = None
    for line in open(os.path.join(*filename), "r"):
        if line.startswith('#'): continue
        values = line.split()
        if not values: continue
        if   values[0] == 'v' : vertices.append( map(float,values[1:4]))
        elif values[0] == 'vn': normals.append(  map(float,values[1:4]))
        elif values[0] == 'vt': texcoords.append(map(float,values[1:3]))
        elif values[0] in ('usemtl','usemat'): material = values[1]
        elif values[0] == 'mtllib': continue
        elif values[0] == 'f':
            face = []
            facetexcoords = []
            facenorms = []
            for v in values[1:]:
                w = v.split('/')
                face.append(int(w[0]))
                if len(w) >= 2 and len(w[1]) > 0: facetexcoords.append(int(w[1]))
                else:                             facetexcoords.append(0)
                if len(w) >= 3 and len(w[2]) > 0: facenorms.append(int(w[2]))
                else:                             facenorms.append(0)
            faces.append((face, facenorms, facetexcoords, material))
    currentcolor = glGetFloatv(GL_CURRENT_COLOR)
    lastmaterial = None
    colorchanged = False
    textures = []
    vertexarrays = []
    normalarrays = []
    texturecoordarrays = []
    tbnvectorarrays = []
    vertexarray = []
    normalarray = []
    texturecoordarray = []
    tbnvectorarray = []
    for face in faces:
        vertexindices, normalindices, texcoordindices, material = face
        if len(vertexindices) != 3: print "Uh oh!  Object does not consist entirely of triangles!"
        facemtl = mtl[material]
        if 'texture_Kd' in facemtl:
            currentmaterial = facemtl['texture_Kd']
            if lastmaterial != currentmaterial:
                lastmaterial = currentmaterial
                vertexarrays.append(vertexarray)
                normalarrays.append(normalarray)
                texturecoordarrays.append(texturecoordarray)
                tbnvectorarrays.append(tbnvectorarray)
                textures.append(currentmaterial)
                vertexarray = []
                normalarray = []
                texturecoordarray = []
                tbnvectorarray = []
        else:
            currentmaterial = facemtl['Kd']
            if lastmaterial != currentmaterial:
                lastmaterial = currentmaterial
##                glColor(*facemtl['Kd'])
                colorchanged = True
        triangle = [[vertices[vertexindices[0]-1],vertices[vertexindices[1]-1],vertices[vertexindices[2]-1]],
                    [texcoords[texcoordindices[0]-1],texcoords[texcoordindices[1]-1],texcoords[texcoordindices[2]-1]],
                    [normals[normalindices[0]-1],normals[normalindices[1]-1],normals[normalindices[2]-1]]]
        tritangents = CalculateTangentArray(triangle)
        for i in xrange(0,3):
            vertexarray.append(triangle[0][i])
            normalarray.append(triangle[2][i])
            texturecoordarray.append(triangle[1][i])
            tbnvectorarray.append(tritangents[i])
    vertexarrays.append(vertexarray)
    normalarrays.append(normalarray)
    texturecoordarrays.append(texturecoordarray)
    tbnvectorarrays.append(tbnvectorarray)
    textures.append(currentmaterial)
    if colorchanged:
        glColor4f(*currentcolor)
    tbnvectorarrays2 = []
    for array in tbnvectorarrays:
        if array == []:
            tbnvectorarrays2.append([])
        else:
            numericarray = Numeric.zeros((len(array),4),'f')
            index = 0
            for element in array:
                numericarray[index,0] = element[0]
                numericarray[index,1] = element[1]
                numericarray[index,2] = element[2]
                numericarray[index,3] = element[3]
                index += 1
            tbnvectorarrays2.append(numericarray.tostring())
    return vertexarrays, normalarrays, texturecoordarrays, tbnvectorarrays2, textures
def Normalize(vec):
    length = hypot(hypot(vec[0],vec[1]),vec[2])
    if length == 0: return [1,0,0]
    return [vec[0]/length,vec[1]/length,vec[2]/length]
def VecSubt(v1,v2):
    return (v1[0]-v2[0],v1[1]-v2[1],v1[2]-v2[2])
def ScVec(s,vec):
    return (s*vec[0],s*vec[1],s*vec[2])
def Dot(v1,v2):
    return (v1[0]*v2[0]) + (v1[1]*v2[1]) + (v1[2]*v2[2])
def Cross(v1,v2):
    return (((v1[1]*v2[2])-(v1[2]*v2[1])),((v1[2]*v2[0])-(v1[0]*v2[2])),((v1[0]*v2[1])-(v1[1]*v2[0])))
def CalculateTangentArray(triangle):
    vertex1 = triangle[0][0]
    vertex2 = triangle[0][1]
    vertex3 = triangle[0][2]
    texcoord1 = triangle[1][0]
    texcoord2 = triangle[1][1]
    texcoord3 = triangle[1][2]
    x1 = vertex2[0]-vertex1[0]
    x2 = vertex3[0]-vertex1[0]
    y1 = vertex2[1]-vertex1[1]
    y2 = vertex3[1]-vertex1[1]
    z1 = vertex2[2]-vertex1[2]
    z2 = vertex3[2]-vertex1[2]
    s1 = texcoord2[0]-texcoord1[0]
    s2 = texcoord3[0]-texcoord1[0]
    t1 = texcoord2[1]-texcoord1[1]
    t2 = texcoord3[1]-texcoord1[1]
    divisor = ((s1*t2)-(s2*t1))
    if divisor == 0:
        divisor = 0.01
    r = 1.0/divisor
    sdir = ((t2*x1-t1*x2)*r,(t2*y1-t1*y2)*r,(t2*z1-t1*z2)*r)
    tdir = ((s1*x2-s2*x1)*r,(s1*y2-s2*y1)*r,(s1*z2-s2*z1)*r)
    tangents = []
    for i in xrange(3):
        n = triangle[2][i]
        t = sdir
        tangent = Normalize(VecSubt(t,ScVec(Dot(n,t),n)))
        if Dot(Cross(n,t),tdir) < 0.0: tangent.append(-1.0)
        else:                          tangent.append( 1.0)
        tangents.append(map(float,tangent))
    return tangents
