from glLibLocals import *
from glLibOBJLoad import *
from glLibShader import *
def glLibTexFullScreenQuad(size="auto"):
    if size == "auto":
        size = glGetFloatv(GL_VIEWPORT)
        size = [int(round(size[2])),int(round(size[3]))]
    glBegin(GL_QUADS)
    glTexCoord2f(0,0);glVertex2f(0,0)
    glTexCoord2f(1,0);glVertex2f(size[0],0)
    glTexCoord2f(1,1);glVertex2f(size[0],size[1])
    glTexCoord2f(0,1);glVertex2f(0,size[1])
    glEnd()
class glLibSphere:
    def __init__(self,size,detail,texture=False):
        self.list = glGenLists(1)
        glNewList(self.list, GL_COMPILE)
        Sphere = gluNewQuadric()
        if texture: gluQuadricTexture(Sphere,GLU_TRUE)
        gluSphere(Sphere,size,detail,detail)
        glEndList()
class glLibObject:
    def __init__(self,path,mipmapping,filtering):
        self.vertices, self.normals, self.texturecoords, self.tbnvectors, self.textures = OBJ(path,mipmapping,filtering)
    def draw(self,shader):
        glLibUseShader(shader)
        
        location = glGetAttribLocation(shader.program,"vert_tangent")
        
        glEnableClientState(GL_VERTEX_ARRAY)
        glEnableClientState(GL_NORMAL_ARRAY)
        glEnableClientState(GL_TEXTURE_COORD_ARRAY)
        if location != -1: glEnableVertexAttribArray(location)

        for i in xrange(len(self.vertices)):
##            print self.vertices
##            glBindTexture(GL_TEXTURE_2D,self.textures[i].texture)
##            shader.pass_texture(self.textures[i])
            if self.vertices[i] != []:
                glVertexPointer(3,GL_FLOAT,0,self.vertices[i])
                glNormalPointer(GL_FLOAT,0,self.normals[i])
                glTexCoordPointer(2,GL_FLOAT,0,self.texturecoords[i])
                if location != -1: glVertexAttribPointer(location,4,GL_FLOAT,GL_FALSE,0,self.tbnvectors[i])
                glDrawArrays(GL_TRIANGLES,0,len(self.vertices[i]))
        
        if location != -1: glDisableVertexAttribArray(location)
        glDisableClientState(GL_TEXTURE_COORD_ARRAY)
        glDisableClientState(GL_NORMAL_ARRAY)
        glDisableClientState(GL_VERTEX_ARRAY)
def glLibGetNormal(t1,t2,t3,flip=False):
    v1 = [t2[0]-t1[0],t2[1]-t1[1],t2[2]-t1[2]]
    v2 = [t3[0]-t1[0],t3[1]-t1[1],t3[2]-t1[2]]
    vx = (v1[1] * v2[2]) - (v1[2] * v2[1])
    vy = (v1[2] * v2[0]) - (v1[0] * v2[2])
    vz = (v1[0] * v2[1]) - (v1[1] * v2[0])
    n = [vx, vy, vz]
    if flip:
        n = [-n[0],-n[1],-n[2]]
    return n
class glLibHeightmapObject:
    def __init__(self,mapdata,texture=False,normals=None,heightscalar=1.0):
        texturemap = []
        normalmap = []
        if type(mapdata) == type(""):
            heightmap = pygame.image.load(mapdata)
            mapdata = []
            for x in xrange(heightmap.get_height()):
                xrow = []
                for y in xrange(heightmap.get_height()):
                    color = heightmap.get_at((x,y))
                    height = color[0]*(1.0/255.0)
                    xrow.append(height)
                mapdata.append(xrow)
        for z in xrange(len(mapdata)):
            normalrow = []
            texturerow = []
            for x in xrange(len(mapdata[0])):
                if normals == GLLIB_VERTEX_NORMALS:
                    Normals = []
                    t = 0
                    flip = True
                    leftpoint = rightpoint = uppoint = downpoint = False
                    normalpoint = (x,  heightscalar*mapdata[z  ][x  ],z  )
                    if x-1 >= 0:                 leftpoint   = (x-1,heightscalar*mapdata[z  ][x-1],z  )#; print "leftpoint", leftpoint
                    if x+1 <= len(mapdata[0])-1: rightpoint  = (x+1,heightscalar*mapdata[z  ][x+1],z  )#; print "rightpoint", rightpoint
                    if z+1 <= len(mapdata)-1:    uppoint     = (x,  heightscalar*mapdata[z+1][x  ],z+1)#; print "uppoint", uppoint
                    if z-1 >= 0:                 downpoint   = (x,  heightscalar*mapdata[z-1][x  ],z-1)#; print "downpoint", downpoint
                    if rightpoint and uppoint:   Normals.append(glLibGetNormal(normalpoint,rightpoint,uppoint,flip))
                    if uppoint and leftpoint:    Normals.append(glLibGetNormal(normalpoint,uppoint,leftpoint,flip))
                    if leftpoint and downpoint:  Normals.append(glLibGetNormal(normalpoint,leftpoint,downpoint,flip))
                    if downpoint and rightpoint: Normals.append(glLibGetNormal(normalpoint,downpoint,rightpoint,flip))
                    xcomp = []; ycomp = []; zcomp = []
                    for n in Normals:
                        xcomp.append(n[0]);ycomp.append(n[1]);zcomp.append(n[2])
                    normal = (sum(xcomp)/len(xcomp),sum(ycomp)/len(ycomp),sum(zcomp)/len(zcomp))
                    l = sqrt((normal[0]**2)+(normal[1]**2)+(normal[2]**2))
                    normal = [normal[0]/l,normal[1]/l,normal[2]/l]
                    normalrow.append(normal)
                if texture != False:
                    xtexcoord = float(x)/(len(mapdata[0])-1)
                    ytexcoord = float(z)/(len(mapdata)-1)
                    texturerow.append((xtexcoord,ytexcoord))
            normalmap.append(normalrow)
            texturemap.append(texturerow)
        self.list = glGenLists(1)
        glNewList(self.list, GL_COMPILE)
        alreadytexturing = glGetBooleanv(GL_TEXTURE_2D)
        if texture:
            glLibSelectTexture(texture)
            glBindTexture(GL_TEXTURE_2D,texture)
        glBegin(GL_QUADS)
        for z in xrange(len(mapdata)-1):
            zrow1 = mapdata[z]
            zrow2 = mapdata[z+1]
            for x in xrange(len(zrow1)-1):
                if normals == GLLIB_FACE_NORMALS:
                    Normals = []
                    Normals.append(glLibGetNormal((x,  heightscalar*mapdata[z  ][x  ],z  ),(x,  heightscalar*mapdata[z+1][x  ],z+1),(x+1,heightscalar*mapdata[z  ][x+1],z  )))
                    Normals.append(glLibGetNormal((x,  heightscalar*mapdata[z+1][x  ],z+1),(x+1,heightscalar*mapdata[z+1][x+1],z+1),(x,  heightscalar*mapdata[z  ][x  ],z  )))
                    Normals.append(glLibGetNormal((x+1,heightscalar*mapdata[z  ][x+1],z  ),(x,  heightscalar*mapdata[z  ][x  ],z  ),(x+1,heightscalar*mapdata[z+1][x+1],z+1)))
                    Normals.append(glLibGetNormal((x+1,heightscalar*mapdata[z+1][x+1],z+1),(x+1,heightscalar*mapdata[z  ][x+1],z  ),(x,  heightscalar*mapdata[z+1][x  ],z+1)))
                    xcomp = []; ycomp = []; zcomp = []
                    for n in Normals:
                        xcomp.append(n[0]);ycomp.append(n[1]);zcomp.append(n[2])
                    normal = (sum(xcomp)/4.0,sum(ycomp)/4.0,sum(zcomp)/4.0)
                    l = sqrt((normal[0]**2)+(normal[1]**2)+(normal[2]**2))
                    normal = [normal[0]/l,normal[1]/l,normal[2]/l]
                    glNormal3f(*normal)
                if normals == GLLIB_VERTEX_NORMALS: glNormal3f(*normalmap[z-1  ][x-1  ])
                if texture: glTexCoord2f(*texturemap[z  ][x  ])
                glVertex3f(x,  heightscalar*zrow1[x  ],z  )
                if normals == GLLIB_VERTEX_NORMALS: glNormal3f(*normalmap[z-1  ][x-1+1])
                if texture: glTexCoord2f(*texturemap[z  ][x+1])
                glVertex3f(x+1,heightscalar*zrow1[x+1],z  )
                if normals == GLLIB_VERTEX_NORMALS: glNormal3f(*normalmap[z-1+1][x-1+1])
                if texture: glTexCoord2f(*texturemap[z+1][x+1])
                glVertex3f(x+1,heightscalar*zrow2[x+1],z+1)
                if normals == GLLIB_VERTEX_NORMALS: glNormal3f(*normalmap[z-1+1][x-1  ])
                if texture: glTexCoord2f(*texturemap[z+1][x  ])
                glVertex3f(x,  heightscalar*zrow2[x  ],z+1)
        glEnd()
        if alreadytexturing: glEnable(GL_TEXTURE_2D)
        glEndList()
    def draw(self):
        glCallList(self.list)
