from glLibLocals import *
from glLibShader import glLibShader
def glLibBlankShader():
    Shader = glLibShader()
    return Shader
def glLibColorShader(color,intensity):
    Shader = glLibShader()
    if   color == GLLIB_RED:    color = [1,0,0]
    elif color == GLLIB_ORANGE: color = [1,0.5,0]
    elif color == GLLIB_YELLOW: color = [1,1,0]
    elif color == GLLIB_GREEN:  color = [0,1,0]
    elif color == GLLIB_BLUE:   color = [0,0,1]
    elif color == GLLIB_PURPLE: color = [0.5,0,1]
    elif color == GLLIB_WHITE:  color = [1,1,1]
    elif color == GLLIB_PINK:  color = [1,0,1]
    elif color == GLLIB_CYAN:  color = [0,1,1]
    color = [color[0]*intensity,color[1]*intensity,color[2]*intensity]
    color = map(str,color)
    Shader.render_equation("color = vec4("+color[0]+","+color[1]+","+color[2]+",1.0);")
    return Shader
def glLibMaterialPhongShader():
    Shader = glLibShader()
    Shader.render_equation("""
    color += ambient_light0_factor        * ambient_color;
    color += diffuse_light0_factor        * diffuse_color;
    color += phong_specular_light0_factor * specular_color;
    """)
    return Shader
def glLibTextureShader():
    Shader = glLibShader()
    Shader.render_equation("color = texture2D(tex1,uv);")
    return Shader
def glLibDepthBufferShader():
    Shader = glLibShader()
    Shader.render_equation("""
    float z = gl_FragCoord.z;// / gl_FragCoord.w;
    float intensity = z/(gl_DepthRange.far-gl_DepthRange.near);
    color = vec4(intensity,intensity,intensity,1.0);""")
    return Shader
def glLibBlurShader():
    Shader = glLibShader()
    Shader.user_variables("""uniform int stage;
    uniform vec2 uvstep;
    uniform int size;
    uniform sampler2D scenebuffertex1, scenebuffertex2;""")
    Shader.render_equation("""
    float fsize = float(size);
    vec4 blursample = vec4(0.0,0.0,0.0,1.0);
    float intensity;
    float sum = 0.0;
    if (stage==1) {
        for (int i=-size;i<size+1;i++) {
            intensity = 1.0 - (abs(float(i))/(fsize+1.0));
            blursample += tex(scenebuffertex1,uv+vec2(float(i)*uvstep.x,0))*intensity;
            sum += intensity;
        }
    }
    if (stage==2) {
        for (int i=-size;i<size+1;i++) {
            intensity = 1.0 - (abs(float(i))/(fsize+1.0));
            blursample += tex(scenebuffertex2,uv+vec2(0,float(i)*uvstep.y))*intensity;
            sum += intensity;
        }
    }
    blursample = vec4(blursample.rgb/sum,1.0);
    color += blursample;
    """)
    return Shader
def glLibRadialBlurShader():
    Shader = glLibShader()
    Shader.user_variables("""uniform vec2 center;
    uniform int size;
    uniform float step;
    uniform sampler2D scenebuffertex;""")
    Shader.render_equation("""
    vec4 blursample = vec4(0.0);
    vec2 diff = normalize(uv-center) * step;
    for (int i=-size; i<size; i++) {
        blursample += tex(scenebuffertex,uv+(diff*float(i)));
    }
    blursample /= float(size);
    color = blursample;
    """)
    return Shader
def glLibThresholdShader():
    Shader = glLibShader()
    Shader.user_variables("""uniform vec4 lowthreshold,highthreshold;
    uniform vec4 replacecolor;
    uniform sampler2D scenebuffertex;""")
    Shader.render_equation("""
    vec4 texsample = tex(scenebuffertex,uv);
    color = replacecolor;
    if (texsample.r <= highthreshold.r && texsample.r >= lowthreshold.r) {
        if (texsample.g <= highthreshold.g && texsample.g >= lowthreshold.g) {
            if (texsample.b <= highthreshold.b && texsample.b >= lowthreshold.b) {
                if (texsample.a <= highthreshold.a && texsample.a >= lowthreshold.a) {
                    color = texsample;
                }
            }
        }
    }
    """)
    return Shader
##def glLibShadowShader():
##    Shader = glLibShader()
##    Shader.user_variables("""
##    varying vec4 ProjCoord1;
##    uniform mat4 texture1,texture2,texture3,texture4,texture5,texture6;""")
##    Shader.render_equation("""
##    color += ambient_light0_factor        * ambient_color;
##    color += diffuse_light0_factor        * diffuse_color;
##    color += blinn_specular_light0_factor * specular_color;
##
##    vec4 ShadowValue = shadow2DProj(shadtex1,ProjCoord1);
##    
##    color *= clamp(ShadowValue+0.5,0.0,1.0);
##    """)
##    Shader.vertex_transform("""
##    ProjCoord1 = texture1*gl_Vertex;
##    """)
##    return Shader
GLLIB_SHADER_BLANK = glLibBlankShader()
GLLIB_SHADER_BLUR = glLibBlurShader()
GLLIB_SHADER_RADIALBLUR = glLibRadialBlurShader()
GLLIB_SHADER_DEPTH = glLibDepthBufferShader()
GLLIB_SHADER_RED = glLibColorShader(GLLIB_RED,1.0)
GLLIB_SHADER_ORANGE = glLibColorShader(GLLIB_ORANGE,1.0)
GLLIB_SHADER_YELLOW = glLibColorShader(GLLIB_YELLOW,1.0)
GLLIB_SHADER_GREEN = glLibColorShader(GLLIB_GREEN,1.0)
GLLIB_SHADER_BLUE = glLibColorShader(GLLIB_BLUE,1.0)
GLLIB_SHADER_PURPLE = glLibColorShader(GLLIB_PURPLE,1.0)
GLLIB_SHADER_WHITE = glLibColorShader(GLLIB_WHITE,1.0)
GLLIB_SHADER_BLACK = glLibColorShader(GLLIB_WHITE,0.0)
GLLIB_SHADER_GREY = GLLIB_SHADER_GRAY = glLibColorShader(GLLIB_WHITE,0.2)
GLLIB_SHADER_TEXTURE = glLibTextureShader()
GLLIB_SHADER_MATERIAL = glLibMaterialPhongShader()
GLLIB_SHADER_THRESHOLD = glLibThresholdShader()
