from glLibLocals import *
from glLibTexturing import *
def CompileShader(source,shaderType):
    shader = glCreateShaderObjectARB(shaderType)
    glShaderSourceARB(shader, source)
    glCompileShaderARB(shader)
    return shader
def glLibUseShader(shader):
    if shader != None:
        try:
            if not shader.compiled:
##                raise glLibError("Shader not compiled!")
                shader.compile()
            else:
                shader = shader.program
        except:
            raise glLibError("Shader not a glLibShader object!")
    else:
        shader = 0
    glUseProgramObjectARB(shader)
class glLibShader:
    def __init__(self):
        self.special1tex1 = None
        self.special1tex2 = None
        self.special2tex1 = None
        self.special3tex1 = None
        self.errors = "Shader not yet compiled!"
        self.compiled = False
        self.renderequation = ""
        self.uservars = ""
        self.verttrans = ""
        self.uvtrans = ""
        self.extfunc = ""
    def get_errors(self):
        return self.errors
    def render_equation(self,renderequation):
        self.renderequation = renderequation
    def user_variables(self,user_vars):
        self.uservars = user_vars
    def vertex_transform(self,verttrans):
        self.verttrans = verttrans
    def uv_transform(self,uvtrans):
        self.uvtrans = uvtrans
    def extension_functions(self,extfuncs):
        self.extfunc = extfuncs
    def compile(self):
        self.source = ShaderSource(self.renderequation,self.uservars,self.verttrans,self.uvtrans,self.extfunc)
        vertex_source = self.source.vert
        fragment_source = self.source.frag
        
        self.program = glCreateProgramObjectARB()
        
        VertexShader = CompileShader(vertex_source,GL_VERTEX_SHADER_ARB)
        glAttachObjectARB(self.program,VertexShader)
        
        FragmentShader = CompileShader(fragment_source,GL_FRAGMENT_SHADER_ARB)
        glAttachObjectARB(self.program,FragmentShader)
        
        glValidateProgramARB(self.program)
        glLinkProgramARB(self.program)
        
        glDeleteObjectARB(VertexShader)
        glDeleteObjectARB(FragmentShader)
        
        self.errors = glGetInfoLogARB(self.program)
        self.compiled = True
    def pass_texture(self,texture):
        glLibSelectTexture(texture)
        active_texture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE0
        glUniform1i(glGetUniformLocation(self.program,"tex"+str(active_texture+1)),active_texture)
    def pass_normalmap(self,texture):
        oldtexture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE0
        glLibActiveTexture(6)
        glLibSelectTexture(texture)
        glUniform1i(glGetUniformLocation(self.program,"normalmap"),6)
        glLibActiveTexture(oldtexture)
    def pass_heightmap(self,texture):
        oldtexture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE0
        glLibActiveTexture(7)
        glLibSelectTexture(texture)
        glUniform1i(glGetUniformLocation(self.program,"heightmap"),7)
        glLibActiveTexture(oldtexture)
    def pass_shadow_texture(self,texture):
        glLibSelectTexture(texture)
        active_texture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE8
        glUniform1i(glGetUniformLocation(self.program,"shadtex"+str(active_texture+1)),active_texture+8)
    def pass_float(self,float,name): glUniform1f(glGetUniformLocation(self.program,name),float)
    def pass_int  (self,int,  name): glUniform1i(glGetUniformLocation(self.program,name),int)
    def pass_bool (self,bool, name): glUniform1f(glGetUniformLocation(self.program,name),bool)
    def pass_vec2 (self,vec2, name): glUniform2f(glGetUniformLocation(self.program,name),vec2[0],vec2[1])
    def pass_vec3 (self,vec3, name): glUniform3f(glGetUniformLocation(self.program,name),vec3[0],vec3[1],vec3[2])
    def pass_vec4 (self,vec4, name): glUniform4f(glGetUniformLocation(self.program,name),vec4[0],vec4[1],vec4[2],vec4[3])
    def pass_mat2 (self,mat2, name): glUniformMatrix2fv(glGetUniformLocation(self.program,name),1,False,mat2)
    def pass_mat3 (self,mat3, name): glUniformMatrix3fv(glGetUniformLocation(self.program,name),1,False,mat3)
    def pass_mat4 (self,mat4, name): glUniformMatrix4fv(glGetUniformLocation(self.program,name),1,False,mat4)
class ShaderSource:
    def __init__(self,renderequation,uservars,verttrans,uvtrans,extfunc):
        #VERTEX SHADER SOURCE GENERATION
        self.vert = """
        varying vec4 Vertex;
        varying vec3 vVertex;
        varying vec3 t;
        varying vec3 b;
        varying vec3 n;
        attribute vec4 vert_tangent;
        """+uservars+"""
        void main() {
            gl_TexCoord[0] = gl_MultiTexCoord0;

            vec4 vertex = gl_Vertex;
            Vertex = gl_Vertex;

            n = normalize(gl_NormalMatrix * gl_Normal);
            t = normalize(gl_NormalMatrix * vert_tangent.xyz);
            b = cross(n, t)*vert_tangent.w;
            vVertex = vec3(gl_ModelViewMatrix*gl_Vertex);

            """+verttrans+"""

            gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
        }"""
        #FRAGMENT SHADER SOURCE GENERATION
        self.frag = """
        varying vec4 Vertex;
        varying vec3 vVertex,t,b,n;
        uniform sampler2D tex1,tex2,tex3,tex4,tex5,tex6;
        uniform sampler2D normalmap,heightmap;
        uniform sampler2DShadow shadtex1,shadtex2,shadtex3,shadtex4,shadtex5,shadtex6;
        uniform mat4 texture1,texture2,texture3,texture4,texture5,texture6;
        vec2 offset, uv;
        vec3 eyevec;
        """+uservars+"""

        vec4 fog(vec4 color)
        {
            float z = gl_FragCoord.z / gl_FragCoord.w;
            z = clamp((z-2.0)/5.0,0.0,1.0);
            float oneminusz = 1.0 - z;
            color = vec4((color[0]*oneminusz)+(1.0*z),(color[1]*oneminusz)+(1.0*z),(color[2]*oneminusz)+(1.0*z),1.0);
            return color;
        }
        vec4 cartoon(vec4 color, int steps)
        {
            float grey = (color.r+color.g+color.g)/3.0;
            float steps2 = float(steps);
            float intensity = float(floor(grey*steps2))/steps2;
            color = vec4(intensity*color.rgb,color.a);
            return color;
        }
        vec4 normal_outline(in vec4 color, in float threshhold, in vec4 setcolor, in float normz)
        {
            if (normz < threshhold) {
                return setcolor;
            }
            return color;
        }
        vec2 parallaxmap(sampler2D heightmap, vec2 texcoords, float scale, bool flip)
        {
            float fDepth = 0.0;
            vec2 vHalfOffset = vec2(0.0,0.0);
            int i = 0;
            if (flip) {
                while (i < 3) {
                    fDepth = fDepth+(texture2D(heightmap,texcoords+vHalfOffset).r) * 0.5;
                    vHalfOffset = normalize(eyevec).xy * fDepth * scale;
                    i += 1;
                }
            }
            else {
                while (i < 3) {
                    fDepth = (fDepth+(1.0-texture2D(heightmap,texcoords+vHalfOffset).r)) * 0.5;
                    vHalfOffset = normalize(eyevec).xy * fDepth * scale;
                    i += 1;
                }
            }
            return vHalfOffset;
        }
        float bestdepth(vec2 ds, sampler2D heightmap, vec2 texcoords, float scale, bool flip, int linear_search_steps, int binary_search_steps)
        {
            float depth_step=1.0/float(linear_search_steps);
            float size=depth_step;
            float depth=0.0;
            float best_depth=1.0;
            if (flip) {
                for (int i=0; i<linear_search_steps-1; i++) {depth += size;vec4 t = texture2D(heightmap,texcoords+(ds*depth));if (best_depth>0.996) {if (depth>=t.r){best_depth=depth;}}}
                depth = best_depth;
                for (int i=0; i<binary_search_steps; i++) {size *= 0.5;vec4 t = texture2D(heightmap,texcoords+(ds*depth));if (depth>=t.r) {best_depth=depth;depth-=2.0*size;}depth += size;}
            }
            else {
                for (int i=0; i<linear_search_steps-1; i++) {depth += size;vec4 t = 1.0-texture2D(heightmap,texcoords+(ds*depth));if (best_depth>0.996) {if (depth>=t.r){best_depth=depth;}}}
                depth = best_depth;
                for (int i=0; i<binary_search_steps; i++) {size *= 0.5;vec4 t = 1.0-texture2D(heightmap,texcoords+(ds*depth));if (depth>=t.r) {best_depth=depth;depth-=2.0*size;}depth += size;}
            }
            return best_depth;
        }
        vec3 parallaxocculsionmap(sampler2D heightmap, vec2 texcoords, float scale, bool flip, int linear, int binary)
        {
            vec2 ds = eyevec.xy*scale;
            
            float best_depth = bestdepth(ds,heightmap,texcoords,scale,flip,linear,binary);
            
            offset = ds*best_depth;

            return vec3(offset.x,offset.y,best_depth);
        }
        //bool shadowtest(sampler2DShadow shadowtexture, vec4 coord, bool outside)
        //{
        //    bool shadowValue = outside;
        //    vec2 coord2d = coord.st/coord.q;
        //    if(coord2d.x<=1.0){if(coord2d.x>=0.0){if(coord2d.y<=1.0){if(coord2d.y>=0.0){
        //        float depth = vec4(texture2D(shadowtexture,coord2d)).x;
        //        shadowValue = (coord.p/coord.q <= depth) ? false : true;
        //    }}}}
        //    return shadowValue;
        //}
        //bool isshadowed(int number, bool outside)
        //{
        //    if      (number==1) { return shadowtest(shadtex1,texture1*Vertex,outside); }
        //    else if (number==2) { return shadowtest(shadtex2,texture2*Vertex,outside); }
        //    else if (number==3) { return shadowtest(shadtex3,texture3*Vertex,outside); }
        //    else if (number==4) { return shadowtest(shadtex4,texture4*Vertex,outside); }
        //    else if (number==5) { return shadowtest(shadtex5,texture5*Vertex,outside); }
        //    else if (number==6) { return shadowtest(shadtex6,texture6*Vertex,outside); }
        //}
        vec4 tex(sampler2D texture, vec2 texcoords)
        {
            return texture2D(texture,texcoords);
        }
        vec4 brightness(vec4 color, float value)
        {
            return vec4(color.rgb*value,color.a);
        }
        """+extfunc+"""
        void main()
        {
            //color
            vec4 color = vec4(0.0,0.0,0.0,1.0);

            //Tex coords
            uv = gl_TexCoord[0].st;
            
            eyevec.x = dot(vVertex, t);
            eyevec.y = dot(vVertex, b);
            eyevec.z = dot(vVertex, n);"""
        #Light
        self.frag += "vec3 tmpVec;"
        for lightnum in xrange(8):
            self.frag += "tmpVec = normalize(gl_LightSource[%i].position.xyz-vVertex);" % (lightnum)
            self.frag += "vec3 LightVec%i;"               % (lightnum)
            self.frag += "LightVec%i.x = dot(tmpVec, t);" % (lightnum)
            self.frag += "LightVec%i.y = dot(tmpVec, b);" % (lightnum)
            self.frag += "LightVec%i.z = dot(tmpVec, n);" % (lightnum)

        self.frag += uvtrans+"""

            //Normal
            vec3 normal = vec3(0.0,0.0,1.0);
            vec3 sample = texture2D(normalmap,uv).rgb;
            vec3 normalmap_normal = normalize(vec3((sample.rg-0.5)*2.0,sample.b));"""
        
        #Color Materials
        if "ambient_color"  in renderequation: self.frag += "vec4 ambient_color =  gl_FrontMaterial.ambient;"
        if "diffuse_color"  in renderequation: self.frag += "vec4 diffuse_color =  gl_FrontMaterial.diffuse;"
        if "specular_color" in renderequation: self.frag += "vec4 specular_color = gl_FrontMaterial.specular;"
        for lightnum in xrange(8):
            #Light Vector
            self.frag += "vec3 lightvec%i = LightVec%i*inversesqrt(dot(LightVec%i,LightVec%i));" % (lightnum,lightnum,lightnum,lightnum)
            
            #Light Attenuation
            self.frag += "float dist%i = length(vec3(vVertex.xyz-gl_LightSource[%i].position.xyz));" % (lightnum,lightnum)
            self.frag += "float light%i_att = 1.0/((gl_LightSource[%i].constantAttenuation)+(gl_LightSource[%i].linearAttenuation*dist%i)+(gl_LightSource[%i].quadraticAttenuation*dist%i*dist%i));" % (lightnum,lightnum,lightnum,lightnum,lightnum,lightnum,lightnum)
##            self.frag += "float light%i_att = 1.0/(0.1+(1.0*dist%i)+(0.0*dist%i*dist%i));" % (lightnum,lightnum,lightnum,lightnum)

            #Light Ambient Factors
            self.frag += "vec4 ambient_light%i_factor = gl_LightSource[%i].ambient;" % (lightnum,lightnum)
            
            #Light Diffuse Factors (Interpolated Vertex Normals)
            self.frag += "vec4 diffuse_light%i_factor = gl_LightSource[%i].diffuse * max(dot(lightvec%i,normal),0.0);" % (lightnum,lightnum,lightnum)
            #Light Diffuse Factors (Normal-map Normals)
            self.frag += "vec4 diffuse_light%i_factor_nm = gl_LightSource[%i].diffuse * max(dot(lightvec%i,normalmap_normal),0.0);" % (lightnum,lightnum,lightnum)

            #Light Specular Factors (Interpolated Vertex Normals)
            self.frag += "float phong_coefficient%i = dot(reflect(-lightvec%i,normal),normalize(-eyevec));" % (lightnum,lightnum)
            self.frag += "float blinn_phong_coefficient%i = dot(normal,normalize(normal-eyevec));" % (lightnum)
            self.frag += "vec4 phong_specular_light%i_factor = gl_LightSource[%i].specular * pow(max(phong_coefficient%i,0.0),gl_FrontMaterial.shininess);" % (lightnum,lightnum,lightnum)
            self.frag += "vec4 blinn_specular_light%i_factor = gl_LightSource[%i].specular * pow(max(blinn_phong_coefficient%i,0.0),gl_FrontMaterial.shininess);" % (lightnum,lightnum,lightnum)
            #Light Specular Factors (Normal-map Normals)
            self.frag += "float phong_coefficient%i_nm = dot(reflect(-lightvec%i,normalmap_normal),normalize(-eyevec));" % (lightnum,lightnum)
            self.frag += "float blinn_phong_coefficient%i_nm = dot(normalmap_normal,normalize(normalmap_normal-eyevec));" % (lightnum)
            self.frag += "vec4 phong_specular_light%i_factor_nm = gl_LightSource[%i].specular * pow(max(phong_coefficient%i_nm,0.0),gl_FrontMaterial.shininess);" % (lightnum,lightnum,lightnum)
            self.frag += "vec4 blinn_specular_light%i_factor_nm = gl_LightSource[%i].specular * pow(max(blinn_phong_coefficient%i_nm,0.0),gl_FrontMaterial.shininess);" % (lightnum,lightnum,lightnum)
            
            #Total Light Factors (Interpolated Vertex Normals)
            self.frag += "vec4 blinn_total_light%i_factor = light%i_att*(ambient_light%i_factor+diffuse_light%i_factor+blinn_specular_light%i_factor);" % (lightnum,lightnum,lightnum,lightnum,lightnum)
            self.frag += "vec4 phong_total_light%i_factor = light%i_att*(ambient_light%i_factor+diffuse_light%i_factor+phong_specular_light%i_factor);" % (lightnum,lightnum,lightnum,lightnum,lightnum)
            #Total Light Factors (Normal-map Normals)
            self.frag += "vec4 blinn_total_light%i_factor_nm = light%i_att*(ambient_light%i_factor+diffuse_light%i_factor_nm+blinn_specular_light%i_factor_nm);" % (lightnum,lightnum,lightnum,lightnum,lightnum)
            self.frag += "vec4 phong_total_light%i_factor_nm = light%i_att*(ambient_light%i_factor+diffuse_light%i_factor_nm+phong_specular_light%i_factor_nm);" % (lightnum,lightnum,lightnum,lightnum,lightnum)

        self.frag += renderequation+"""

            //clamp color
            color = clamp(color,0.0,1.0);
            
            //set color
            gl_FragColor = color;
        }"""
